#!/usr/bin/env python3

# Deployment - Test Module verifying deb package installation

import glob
import os
import shutil
import time

from common.appath import APPath
from common.shellhelper import ShellHelper
from distutils.dir_util import copy_tree


# ---------- Module Test Fixtures -----------
# Run once for each module
def setup_module(module):
    setup_test_data()


def teardown_module(module):
    cleanup_test_data()


# This is run before each test case function is executed. Have here for enabling later.
def setup_function(function):
    pass


def teardown_function(function):
    pass


# ---------- End Fixtures -----------

# Deb Test helpers
def setup_test_data():
    print("Copying test packages to packed images dir")
    try:
        copy_tree("./deployment/test_data/", "/ap/app/packedimages/")
    except shutil.Error as e:
        print('Test setup error :' + str(e))


def cleanup_test_data():
    try:
        for path in glob.glob("/ap/app/packedimages/TEST_*"):
            print('Removing', path)
            shutil.rmtree(path)
    except OSError as e:
        print('Test setup error :' + str(e))


def remove_pkgs(pkg_list):
    for pkg in pkg_list:
        ret, result = ShellHelper.run_cmd_output("apt-get -qq -y -f --purge remove " + pkg)
        assert ret == True, "Could not remove " + pkg


def verify_pkg_ver(pkgname, pkgver):
    ret, result = ShellHelper.run_cmd_output("dpkg-query --showformat='${Version}' --show " + pkgname)
    assert result == pkgver, "Package version did not match target version: " + pkgname + "=" + pkgver


def verify_commands(cmd_list):
    for cmd in cmd_list:
        ret, result = ShellHelper.run_cmd_output(cmd)
        assert ret == True, "Verifying command failed: " + cmd


def wait_for_completion(servicelist, test_svclist):
    print("Monitor service manager folder for error package.")
    for i in range(0, 60):
        time.sleep(2)
        # Check if an error file is generated for this package.
        assert os.path.isfile(APPath.ServiceManagerDataRoot + test_svclist + '.err') == False

        # Check if active_servicelist.ini is updated. processing completed.
        if servicelist.verify_active_list(test_svclist):
            break


# Write debian variable file in yaml format.
def write_deb_var_file(filepath, pkg_and_ver):
    content = """---
# Variables for installing debian packages

# TEST: List of packages to be installed and their version. Version can be 'latest' or a
# specific version
debian_pkgs:"""
    with open(filepath, 'w+') as f:
        f.write(content)
        for k, v in pkg_and_ver.items():
            f.write("\n    - name: " + k)
            f.write("\n      version: " + v + "\n")


# Test Cases. Should start with test_ to be picked by pytest.
def test_deploy_latest(apdeployment, servicemanager, servicelist):
    # Test installing latest, pkg with specific versions and already installed packages.
    pkg_and_ver = {'finger': 'latest',
                   'cowsay': 'latest',
                   'sl': 'latest',
                   'rig': 'latest',
                   'python-ogg': 'latest'}

    # Pause system deployment pipeline
    apdeployment.pause()

    # setup_test_data()
    test_svclist = 'TEST_DEB.csv'
    servicelist.update_list(test_svclist)

    var_file = APPath.APPackageFolder + test_svclist + '/install_deb_pkgs/vars/deb_list_var.yml'
    write_deb_var_file(var_file, pkg_and_ver)

    # Remove the packages
    pkg_list = ['finger',
                'cowsay',
                'sl',
                'rig',
                'python-ogg']

    remove_pkgs(pkg_list)

    print("Starting service manager to process the package")
    servicemanager.restart()

    wait_for_completion(servicelist, test_svclist)
    servicemanager.restore_state()

    print("Confirm the debian packages are installed")
    commands = ['finger',
                '/usr/games/cowsay MOOOOO',
                '/usr/bin/rig',
                'file /usr/games/sl']
    verify_commands(commands)

    remove_pkgs(pkg_list)


def test_deploy_specific_version(apdeployment, servicemanager, servicelist):
    # Test installing specific versions of packages
    pkg_and_ver = {  # Specific version of the package
        'fortune-mod': '1:1.99.1-7',
        'figlet': '2.2.5-2',
        'toilet': '0.3-1'}

    # Pause system deployment pipeline
    apdeployment.pause()

    # setup_test_data()
    test_svclist = 'TEST_DEB.csv'
    servicelist.update_list(test_svclist)

    var_file = APPath.APPackageFolder + test_svclist + '/install_deb_pkgs/vars/deb_list_var.yml'
    write_deb_var_file(var_file, pkg_and_ver)

    # Remove the packages
    pkg_list = ['fortune-mod',
                'figlet',
                'toilet']

    remove_pkgs(pkg_list)

    print("Starting service manager to process the package")
    servicemanager.restart()

    wait_for_completion(servicelist, test_svclist)
    servicemanager.restore_state()

    print("Confirm the debian packages are installed")
    # Package installed or timed out.
    commands = ['/usr/games/fortune',
                'which figlet']
    verify_commands(commands)

    # Verify specific target versions of the pkgs are installed
    verify_pkg_ver("fortune-mod", "1:1.99.1-7")
    verify_pkg_ver("toilet", "0.3-1")
    verify_pkg_ver("figlet", "2.2.5-2")

    remove_pkgs(pkg_list)


def test_deploy_already_installed(apdeployment, servicemanager, servicelist):
    # Test installing packages that were already installed.
    pkg_and_ver = {'adduser': 'latest',
                   'at': 'latest',
                   'bash': 'latest',
                   'linux-generic': 'latest',
                   'linux-headers-generic': 'latest'}

    # Pause system deployment pipeline
    apdeployment.pause()

    # setup_test_data()
    test_svclist = 'TEST_DEB.csv'
    servicelist.update_list(test_svclist)

    var_file = APPath.APPackageFolder + test_svclist + '/install_deb_pkgs/vars/deb_list_var.yml'
    write_deb_var_file(var_file, pkg_and_ver)

    print("Starting service manager to process the package")
    servicemanager.restart()

    wait_for_completion(servicelist, test_svclist)

    servicemanager.restore_state()

    print("Confirm the debian packages are there")
    commands = ['which adduser',
                'which at']
    verify_commands(commands)
